using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Data;
using Syncfusion.Windows.Forms.Grid;

namespace Forum_DragColOnFirstClick
{
	
	public class DragColHeaderGrid : GridDataBoundGrid
	{
		public DragColHeaderGrid()
		{
			this.AllowDragSelectedCols = false;
			this.AllowSelection &= ~GridSelectionFlags.Column;
			this.SortBehavior = GridSortBehavior.SingleClick;
		}

		private Point mouseDownPt = Point.Empty;
		private bool inColMove = false;
		private int sourceCol = -1;
		private int targetCol = -1;
		private int targetRow = -1;

		protected override void OnMouseDown(System.Windows.Forms.MouseEventArgs e)
		{
			this.mouseDownPt = new Point(e.X, e.Y);
			base.OnMouseDown(e);
		}

		protected override void OnMouseMove(System.Windows.Forms.MouseEventArgs e)
		{
			if(e.Button == MouseButtons.Left)
			{
				if(!inColMove)
				{
					Point pt = new Point(e.X, e.Y);
					if(Math.Abs(e.X - mouseDownPt.X) > SystemInformation.DragSize.Width / 2
						|| Math.Abs(e.Y - mouseDownPt.Y) > SystemInformation.DragSize.Height / 2)
					{
						int row;
						if(this.PointToRowCol(mouseDownPt, out row, out sourceCol) 
							&& row == 0 && sourceCol > 0)
						{

							Console.WriteLine("start move at {0}", sourceCol);
							inColMove = true;
						}
					}
				}
				else
				{
					this.PointToRowCol(new Point(e.X, e.Y), out targetRow, out targetCol);
				}
				
			}
			base.OnMouseMove(e);
		}

		protected override void OnMouseUp(System.Windows.Forms.MouseEventArgs e)
		{
			if(inColMove)
			{
				
				if(targetCol > 0 && targetRow == 0 && sourceCol > 0)
				{
					Console.WriteLine("move {0} to {1}", sourceCol, targetCol);
					this.Model.Cols.MoveRange(sourceCol, 1, targetCol);
				}
				inColMove = false;
			}
			base.OnMouseUp(e);
		}

		protected override void OnSetCursor(ref System.Windows.Forms.Message m)
		{
			if(inColMove)
			{
				if(targetCol > 0 && targetRow == 0)
					Cursor.Current = GridCursors.DragSelectionCursor;
				else
					Cursor.Current = Cursors.No;
			}
			else
				base.OnSetCursor(ref m);
		}
	}
}
